using System.Collections;
using System.Collections.Generic;
using UnityEngine;
/*The collision script attached to monitor haptics on the specific body part.
 * The collisions are registerd on hitting a non kinematic rigibody.
 * If you do not want physics to be applied to the rigidbody constrain the position and rotation and attach the ConstrainedRigidBodyDynamics script to the rigidbody.
 */
public class ShockwaveIndividualCollider : MonoBehaviour
{
    [Tooltip("Index of the haptic(0-71) ")]
    public int colliderIndex;
    Vector3 velocity, lastPosition;
    Quaternion angularVelocity, quat;
    ShockwaveManager manager;
    Shockwave_SteamVR_Haptics steamVR_manager;
    float boxDiagonal=1;
    Vector3 center;
    // Start is called before the first frame update
    void Start()
    {
        
       
        boxDiagonal = GetComponent<CapsuleCollider>().bounds.size.magnitude / 2;
        center= GetComponent<CapsuleCollider>().bounds.center;
        manager = ShockwaveManager.Instance;
        steamVR_manager = Shockwave_SteamVR_Haptics.Instance;
    }

    void FixedUpdate()
    {
        velocity = (transform.position - lastPosition) / Time.fixedDeltaTime;
        lastPosition = transform.position;
        angularVelocity = (transform.rotation * quat);//This is change not angular velocity.but subtracting the identity quaternion operation per delta time is the angular velocity.
        quat = Quaternion.Inverse(transform.rotation);
    }
    Vector3 GetVelocityAtPoint(Vector3 worldPos)
    {
        Vector3 relativePos = worldPos - transform.position;
        return velocity + (angularVelocity * (relativePos) - relativePos) / Time.fixedDeltaTime;
    }

    private void OnCollisionEnter(Collision collision)
    {


        ContactPoint[] contacts = new ContactPoint[collision.contactCount];
        collision.GetContacts(contacts);

     
        float impactPerPoint = 0;
        center = GetComponent<CapsuleCollider>().bounds.center;
        ConstrainedRigidBodyDynamics con = collision.transform.GetComponent<ConstrainedRigidBodyDynamics>();
        if (con == null)
        {
            impactPerPoint = collision.impulse.magnitude / collision.contactCount;//For Unconstrained RigidBodies
           
            foreach (ContactPoint contact in contacts)
            {
                if (manager != null)
                {
                    if (impactPerPoint > 1)
                        manager.sendHapticsPulse(colliderIndex, impactPerPoint * 2, 500);
                    else if (impactPerPoint > 0.5f)
                        manager.sendHapticsPulse(colliderIndex, impactPerPoint * 2, impactPerPoint * 250);
                    else
                        manager.sendHapticsPulse(colliderIndex, impactPerPoint * 2.7f, 250);
                }
                
            }
        }
        else
        {

            foreach (ContactPoint contact in contacts)
            {
                Vector3 vel = con.GetVelocityAtPoint(contact.point) - GetVelocityAtPoint(contact.point);//Relative Velocity
                float posDistance = (center - contact.point).magnitude / boxDiagonal; //Scaled Distance
                impactPerPoint = collision.transform.GetComponent<Rigidbody>().mass * Mathf.Abs(Vector3.Dot(contact.normal, vel));
              if(manager!=null)
                manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);
               if (steamVR_manager != null)
                    steamVR_manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);

            }
        }
        


    }
    private void OnCollisionStay(Collision collision)
    {
        ContactPoint[] contacts = new ContactPoint[collision.contactCount];
        collision.GetContacts(contacts);

        center = GetComponent<CapsuleCollider>().bounds.center;
        float impactPerPoint = 0;
        if (manager != null)
            impactPerPoint = manager.MaximumIntensity* Time.deltaTime/0.01f;
        else if (steamVR_manager != null)
            impactPerPoint = steamVR_manager.IntensityInsideCollider * steamVR_manager.MaximumIntensity * Time.deltaTime / 0.01f;
        foreach (ContactPoint contact in contacts)
        {
            float posDistance = (center - contact.point).magnitude / boxDiagonal;
           if (manager != null)
                manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);
       if (steamVR_manager != null)
                steamVR_manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);
        }

    }
    private void OnParticleCollision(GameObject partSystem)
    {

        ParticleSystem particles = partSystem.GetComponent<ParticleSystem>();
        List<ParticleCollisionEvent> collisionEvents = new List<ParticleCollisionEvent>();
        int contacts = particles.GetCollisionEvents(gameObject, collisionEvents);

        
        float impactPerPoint = 0;

        center = GetComponent<CapsuleCollider>().bounds.center;

        for (int i=0;i<contacts;i++)
            {
               
                Vector3 vel = collisionEvents[i].velocity - GetVelocityAtPoint(collisionEvents[i].intersection);//Relative Velocity
                float posDistance = (center - collisionEvents[i].intersection).magnitude / boxDiagonal; //Scaled Distance
            float mass = 0.001f;
            if (partSystem.GetComponent<Shockwave_ParticleMass>() != null)
                mass=partSystem.GetComponent<Shockwave_ParticleMass>().massOfParticle;
                impactPerPoint =  Mathf.Abs(Vector3.Dot(collisionEvents[i].normal, vel))*manager.MaximumIntensity *mass;//Here you can use your own factor instead of mass or attach a script to the particle system to sepcify their mass.
          
            if (manager != null)
                manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);
            else if (steamVR_manager != null)
                steamVR_manager.sendHapticsWithPosition(colliderIndex, impactPerPoint, posDistance);

        }
           
    
       



    }
}
